import { ref, type App, type Directive } from 'vue'
import type { SFCInstallWithContext, SFCWithInstall } from './helper.typescript'
import request from './request'
import { type LotButtonItem } from '../types/all'

export const createNamespace = (name: String) => {
  return {
    name: `zfz-${name}`,
    gcn: (className: String = '', isAddName: Boolean = true) => {
      return 'zfz-' + (isAddName ? name : '') + (isAddName && className ? '-' : '') + className
    }
  }
}

export const installByComponent = <T, E extends Record<string, any>>(main: T, extra?: E) => {
  ; (main as SFCWithInstall<T>).install = (app: App): void => {
    for (const comp of [main, ...Object.values(extra ?? {})]) {
      app.component(comp.name, comp)
    }
  }

  if (extra) {
    for (const [key, comp] of Object.entries(extra)) {
      ; (main as any)[key] = comp
    }
  }
  return main as SFCWithInstall<T> & E
}

export const installByFunction = <T>(fn: T, name: string) => {
  ; (fn as SFCWithInstall<T>).install = (app: App) => {
    ; (fn as SFCInstallWithContext<T>)._context = app._context
    app.config.globalProperties[name] = fn
  }

  return fn as SFCInstallWithContext<T>
}

export const installByDirective = <T extends Directive>(directive: T, name: string) => {
  ; (directive as SFCWithInstall<T>).install = (app: App): void => {
    app.directive(name, directive)
  }

  return directive as SFCWithInstall<T>
}

export const useInputComponentProps = function () {
  return {
    placeholder: { type: String, default: '请输入' },
    label: { type: String },
    labelVisible: { type: Boolean, default: true },
    text: { type: String },
    rules: { type: Object },
    input: { type: Boolean, default: true },
    clearable: { type: Boolean, default: true },
    disabled: { type: Boolean }
  }
}

export function useQueryComponentProps() {
  return {
    placeholder: { type: String, default: '请输入' },
    label: { type: String },
    labelVisible: { type: Boolean, default: true },
    text: { type: String },
    rules: { type: Object },
    clearable: { type: Boolean, default: true },
    disabled: { type: Boolean }
  }
}

export function useGetDataSourceResult(
  dataDriver: string | undefined,
  optionList: any[] | undefined,
  webapiUrl: string | undefined,
  webapiType: string | undefined,
  webapiData: object | undefined
) {
  const dataSource = ref([])
  const dataStatus = ref('success')
  const dataErrMsg = ref('')

  if (dataDriver === 'options') {
    dataSource.value = optionList as any
  } else if (dataDriver === 'webapi' && webapiType === 'post') {
    dataStatus.value = 'loading'
    dataErrMsg.value = '加载中...'
    request.post({
      url: webapiUrl,
      data: webapiData
    }).then((res: any) => {
      dataStatus.value = res.code === 0 ? 'success' : 'error'
      dataErrMsg.value = res.code === 0 ? '' : res.msg
      if (res.code === 0) {
        dataSource.value = res.data.rows
      }
    })
  } else if (dataDriver === 'webapi' && webapiType === 'get') {
    dataStatus.value = 'loading'
    dataErrMsg.value = '加载中...'
    request.get({
      url: webapiUrl,
      data: webapiData
    }).then((res: any) => {
      dataStatus.value = res.code === 0 ? 'success' : 'error'
      dataErrMsg.value = res.code === 0 ? '' : res.msg
      if (res.code === 0) {
        dataSource.value = res.data.rows
      }
    })
  }

  return {
    dataSource,
    dataStatus,
    dataErrMsg
  }
}

export function useCommonListPage({
  queryOrder = { field: 'id', type: 1, items: [] },
  webapiUrl,
  webapiType = 'post',
  columnList = [],
  lotButtons = [],
  rowButtons = [],
  initBefore
}: {
  queryOrder?: object,
  webapiUrl?: string,
  webapiType?: 'get' | 'post',
  columnList?: Array<any>,
  lotButtons?: Array<LotButtonItem>,
  rowButtons?: [],
  initBefore?: Function
}) {
  const queryWhere = ref({} as any)
  const chooseMode = ref(0)
  const lotBtnCode = ref('')
  const dataSource = ref([] as Array<any>)
  const dataStatus = ref('success')
  const dataErrMsg = ref('')
  const dataPaging = ref({ pageIndex: 1, pageSize: 20, dataCount: 0, pageCount: 0 })
  const treeEntrys = ref([])
  const initDataFn = () => {
    if (!webapiUrl) {
      dataStatus.value = 'error'
      dataErrMsg.value = '查询地址未配置'
      dataSource.value = []
      dataPaging.value.dataCount = 0
      dataPaging.value.pageCount = 0
      return
    }
    dataStatus.value = 'loading'
    dataErrMsg.value = '加载中...'
    request.method({
      url: webapiUrl,
      type: webapiType,
      data: queryWhere.value
    }).then((res: any) => {
      dataStatus.value = res.code === 0 ? 'success' : 'error'
      dataErrMsg.value = res.code === 0 ? '' : res.msg
      dataSource.value = []
      if (res.code === 0) {
        if (res.data.rows && res.data.rows.length > 0) {
          for (let i = 0; i < res.data.rows.length; i++) {
            dataSource.value.push(res.data.rows[i])
          }
        }
      }
    })
  }

  return {
    pageSchema: 'ListPage',
    chooseMode,
    lotBtnCode,
    queryWhere: queryWhere,
    queryOrder,
    dataSource,
    dataStatus,
    dataErrMsg,
    dataPaging,
    columnList,
    lotButtons,
    rowButtons,
    treeEntrys,
    configInfo: {
      idField: 'id',
      treeParentIdField: 'parentId',
      treeLabelField: 'label',
      treeChildCountField: 'childCount',
      treeQueryParentIdField: 'parentId'
    },
    initBefore,
    initDataFn
  }
}

export function useCommonEditPage({
  dataDriver = 'webapi',
  webapiUrl,
  webapiType = 'post',
  initDataFn
}: {
  dataDriver: 'webapi' | 'function',
  webapiUrl?: string,
  webapiType: 'get' | 'post',
  initDataFn?: Function
}) {
  const queryWhere = ref({} as any)
  const dataSource = ref([])
  const dataStatus = ref('success')
  const dataErrMsg = ref('')
  let initDataMd = dataDriver === 'function' ? initDataFn : () => { }
  if (dataDriver === 'webapi') {
    initDataMd = () => {
      const dataResult = useGetDataSourceResult('webapi', undefined, webapiUrl, webapiType, queryWhere)
      dataSource.value = dataResult.dataSource.value
      dataStatus.value = dataResult.dataStatus.value
      dataErrMsg.value = dataResult.dataErrMsg.value
    }
  }
  return {
    pageSchema: 'EditPage',
    queryWhere: queryWhere,
    dataSource,
    dataStatus,
    dataErrMsg,
    initDataFn: initDataMd
  }
}
